@@ -1,6 +1,7 @@
-#pylint: disable=invalid-name
-#pylint: disable=too-many-locals
-#pylint: disable=too-many-arguments
+# pylint: disable=invalid-name
+# pylint: disable=too-many-locals
+# pylint: disable=too-many-arguments
+# pylint: disable=too-many-statements
 """
 A module for finding the beam size in an monochrome image.
 
@@ -8,18 +9,25 @@ Simple and fast calculation of beam sizes from a single monochrome image based
 on the ISO 11146 method of variances.  Some effort has been made to make
 the algorithm less sensitive to background offset and noise.
 
-Finding the center and dimensions of a good beam image::
+Finding the center and dimensions of a monochrome image of a beam is simple::
 
     import imageio
+    import numpy as np
     import laserbeamsize as lbs
 
     beam = imageio.imread("t-hene.pgm")
     x, y, dx, dy, phi = lbs.beam_size(beam)
 
-    print("The beam center is at (%.0f, %.0f)" % (x,y))
+    print("The center of the beam ellipse is at (%.0f, %.0f)" % (x,y))
     print("The ellipse diameter (closest to horizontal) is %.0f pixels" % dx)
     print("The ellipse diameter (closest to   vertical) is %.0f pixels" % dy)
-    print("The ellipse is rotated %.0f° from the horizontal" % (phi*180/3.1416))
+    print("The ellipse is rotated %.0f° ccw from the horizontal" % (phi*180/3.1416))
+
+    ellipticity = dy/dx
+    if ellipticity>0.87:
+        print()
+        print("The beam is circular because the ellipticity=%.2f > 0.87")
+        print("The circular beam diameter is %.0f pixels" % np.sqrt((dx**2+dy**2)/2))
 """
 
 import matplotlib.pyplot as plt
@@ -43,11 +51,15 @@ __all__ = ('subtract_image',
            'ellipse_arrays',
            'elliptical_mask',
            'plot_image_and_ellipse',
+           'major_axis_arrays',
+           'minor_axis_arrays',
+           'visual_report',
            )
 
+
 def rotate_points(x, y, x0, y0, phi):
     """
-    Rotate x and y around designated center.
+    Rotate x and y around designated center (x0,y0).
 
     Args:
         x: x-values of point or array of points to be rotated
@@ -74,6 +86,123 @@ def rotate_points(x, y, x0, y0, phi):
     return xf, yf
 
 
+def values_along_line(image, x0, y0, x1, y1, N=50):
+    """
+    Return x, y, z, and distance values between (x0,y0) and (x1,y1).
+
+    This creates four arrays::
+        x: index of horizontal pixel values along line
+        y: index of vertical pixel values along line
+        z: image values at each of the x,y positions
+        s: distance from start of minor axis to x,y position
+
+    Args:
+        image: the image to work with
+        xc: horizontal center of beam
+        yc: vertical center of beam
+        dx: ellipse diameter for axis closest to horizontal
+        dy: ellipse diameter for axis closest to vertical
+        phi: angle that elliptical beam is rotated [radians]
+        diameters:
+    Returns:
+        x, y, z, and s arrays
+    """
+    d = np.sqrt((x1-x0)**2 + (y1-y0)**2)
+    s = np.linspace(0, 1, N)
+
+    x = x0 + s * (x1-x0)
+    y = y0 + s * (y1-y0)
+
+    xx = x.astype(int)
+    yy = y.astype(int)
+
+    zz = image[yy, xx]
+
+    return xx, yy, zz, (s-0.5)*d
+
+
+def major_axis_arrays(image, xc, yc, dx, dy, phi, diameters=3):
+    """
+    Return x, y, z, and distance values along semi-major axis.
+
+    This creates four arrays::
+        x: index of horizontal pixel values along line
+        y: index of vertical pixel values along line
+        z: image values at each of the x,y positions
+        s: distance from start of minor axis to x,y position
+
+    Args:
+        image: the image to work with
+        xc: horizontal center of beam
+        yc: vertical center of beam
+        dx: ellipse diameter for axis closest to horizontal
+        dy: ellipse diameter for axis closest to vertical
+        phi: angle that elliptical beam is rotated [radians]
+        diameters:
+    Returns:
+        x, y, z, and s arrays
+    """
+    v, h = image.shape
+
+    if dx > dy:
+        rx = diameters * dx / 2
+        left = max(xc-rx, 0)
+        right = min(xc+rx, h-1)
+        x = np.array([left, right])
+        y = np.array([yc, yc])
+        xr, yr = rotate_points(x, y, xc, yc, phi)
+    else:
+        ry = diameters * dy / 2
+        top = max(yc-ry, 0)
+        bottom = min(yc+ry, v-1)
+        x = np.array([xc, xc])
+        y = np.array([top, bottom])
+        xr, yr = rotate_points(x, y, xc, yc, phi)
+
+    return values_along_line(image, xr[0], yr[0], xr[1], yr[1])
+
+
+def minor_axis_arrays(image, xc, yc, dx, dy, phi, diameters=3):
+    """
+    Return x, y, z, and distance values along semi-minor axis.
+
+    This creates four arrays::
+        x: index of horizontal pixel values along line
+        y: index of vertical pixel values along line
+        z: image values at each of the x,y positions
+        s: distance from start of minor axis to x,y position
+
+    Args:
+        image: the image to work with
+        xc: horizontal center of beam
+        yc: vertical center of beam
+        dx: ellipse diameter for axis closest to horizontal
+        dy: ellipse diameter for axis closest to vertical
+        phi: angle that elliptical beam is rotated [radians]
+        diameters:
+    Returns:
+        x, y, z, and s arrays
+    """
+    v, h = image.shape
+
+    if dx <= dy:
+        rx = diameters * dx / 2
+        left = max(xc-rx, 0)
+        right = min(xc+rx, h-1)
+        x = np.array([left, right])
+        y = np.array([yc, yc])
+        xr, yr = rotate_points(x, y, xc, yc, phi)
+    else:
+        ry = diameters * dy / 2
+        top = max(yc-ry, 0)
+        bottom = min(yc+ry, v-1)
+        x = np.array([xc, xc])
+        y = np.array([top, bottom])
+        xr, yr = rotate_points(x, y, xc, yc, phi)
+
+    return values_along_line(image, xr[0], yr[0], xr[1], yr[1])
+
+
 def subtract_image(original, background):
     """
     Subtract background from original image.
@@ -526,7 +655,7 @@ def beam_test_image(h, v, xc, yc, dx, dy, phi, noise=0, max_value=255):
     y, x = np.ogrid[:v, :h]
 
     scale = max_value - 3 * noise
-    image0 = scale * np.exp(-2*(x-xc)**2/rx**2 -2*(y-yc)**2/ry**2)
+    image0 = scale * np.exp(-2*(x-xc)**2/rx**2 - 2*(y-yc)**2/ry**2)
 
     image1 = rotate_image(image0, xc, yc, phi)
 
@@ -657,8 +786,8 @@ def draw_beam_figure():
 
     plt.subplots(1, 1, figsize=(6, 6))
 
-    #If the aspect ratio is not `equal` then the major and minor radii
-    #do not appear to be orthogonal to each other!
+    # If the aspect ratio is not `equal` then the major and minor radii
+    # do not appear to be orthogonal to each other!
     plt.axes().set_aspect('equal')
 
     xp, yp = ellipse_arrays(xc, yc, dx, dy, theta)
@@ -689,4 +818,86 @@ def draw_beam_figure():
     plt.xlim(-30, 30)
     plt.ylim(30, -30)  # inverted to match image coordinates!
     plt.axis('off')
-    plt.show()
+
+
+def visual_report(image, title='Original', **kwargs):
+    """
+    Create a visual report for image fitting.
+
+    Args:
+        image: 2D image of laser beam
+        title: optional title for report
+    Returns:
+        nothing
+    """
+    beamsize_keys = ['mask_diameters', 'corner_fraction', 'nT', 'max_iter']
+    bs_args = dict((k, kwargs[k]) for k in beamsize_keys if k in kwargs)
+
+    corner_keys = ['corner_fraction', 'nT']
+    c_args = dict((k, kwargs[k]) for k in corner_keys if k in kwargs)
+
+    xc, yc, dx, dy, phi = beam_size(image, **bs_args)
+    working_image = corner_subtract(image, **c_args)
+    min_ = image.min()
+    max_ = image.max()
+    rx = dx/2
+    ry = dy/2
+    vv, hh = image.shape
+
+    plt.subplots(2, 2, figsize=(12, 12))
+    plt.subplots_adjust(right=1.0)
+
+    # original image
+    plt.subplot(2, 2, 1)
+    plt.imshow(image, cmap='gist_ncar')
+    plt.title(title)
+    plt.clim(min_, max_)
+    plt.colorbar(fraction=0.046, pad=0.04)
+    plt.xlabel('Pixels')
+    plt.ylabel('Pixels')
+    plt.ylim(vv, 0)
+    plt.xlim(0, hh)
+
+    # working image
+    plt.subplot(2, 2, 2)
+    plt.imshow(working_image, cmap='gist_ncar')
+    xp, yp = axes_arrays(xc, yc, dx, dy, phi)
+    plt.plot(xp, yp, ':y')
+    xp, yp = rotated_rect_arrays(xc, yc, dx, dy, phi)
+    plt.plot(xp, yp, 'w')
+
+    plt.title('Found (%d, %d), dx=%d, dy=%d' % (xc, yc, dx, dy))
+    plt.clim(min_, max_)
+    plt.colorbar(fraction=0.046, pad=0.04)
+    plt.xlabel('Pixels')
+    plt.ylabel('Pixels')
+    plt.ylim(vv, 0)
+    plt.xlim(0, hh)
+
+    # plot of values along semi-major axis
+    plt.subplot(2, 2, 3)
+    _, _, z, s = major_axis_arrays(image, xc, yc, dx, dy, phi)
+    a = np.sqrt(2/np.pi)/rx * np.sum(z)*abs(s[1]-s[0])
+    h = a*np.exp(-2)
+    plt.plot(s, z, 'or', label='data')
+    plt.plot(s, a*np.exp(-2*(s/rx)**2), ':k', label='fitted')
+    plt.xlabel('Distance from Center (pixels)')
+    plt.ylabel('Pixel Intensity Along Semi-Major Axis')
+    plt.title('Semi-Major Axis')
+    plt.annotate('', (-rx, h), (rx, h), arrowprops=dict(arrowstyle="<->"))
+    plt.text(0, 1.1*h, 'dx=%.1f' % dx, va='bottom', ha='center')
+    plt.text(0, a, '  Gaussian')
+
+    # plot of values along semi-minor axis
+    plt.subplot(2, 2, 4)
+    _, _, z, s = minor_axis_arrays(image, xc, yc, dx, dy, phi)
+    a = np.sqrt(2/np.pi)/ry * np.sum(z)*abs(s[1]-s[0])
+    h = a*np.exp(-2)
+    plt.plot(s, z, 'or', label='data')
+    plt.plot(s, a*np.exp(-2*(s/ry)**2), ':k', label='fitted')
+    plt.xlabel('Distance from Center (pixels)')
+    plt.ylabel('Pixel Intensity Along Semi-Minor Axis')
+    plt.title('Semi-Minor Axis')
+    plt.annotate('', (-ry, h), (ry, h), arrowprops=dict(arrowstyle="<->"))
+    plt.text(0, 1.1*h, 'dy=%.1f' % dy, va='bottom', ha='center')
+    plt.text(0, a, '  Gaussian')
